@@ -153,7 +153,7 @@ out:
  * 2. Open step.
  *    The one who creates the file (the 'master') creates one,
  *    then creates one more unix socket (transport) and sends the
- *    created file over this socket to the other recepients.
+ *    created file over this socket to the other recipients.
  * 3. Receive step.
  *    Those, who wait for the file to appear, receive one via
  *    the transport socket, then close the socket and dup() the
@@ -431,6 +431,7 @@ static int dump_one_file(struct parasite_ctl *ctl, int fd, int lfd, struct fd_op
 {
 	struct fd_parms p = FD_PARMS_INIT;
 	const struct fdtype_ops *ops;
+	struct fd_link link;
 
 	if (fill_fd_params(ctl, fd, lfd, opts, &p) < 0) {
 		pr_err("Can't get stat on %d\n", fd);
@@ -471,8 +472,6 @@ static int dump_one_file(struct parasite_ctl *ctl, int fd, int lfd, struct fd_op
 	}
 
 	if (S_ISREG(p.stat.st_mode) || S_ISDIR(p.stat.st_mode)) {
-		struct fd_link link;
-
 		if (fill_fdlink(lfd, &p, &link))
 			return -1;
 
@@ -495,7 +494,15 @@ static int dump_one_file(struct parasite_ctl *ctl, int fd, int lfd, struct fd_op
 		return do_dump_gen_file(&p, lfd, ops, img);
 	}
 
-	return dump_unsupp_fd(&p, lfd, img, "unknown", NULL);
+	/*
+	 * For debug purpose -- at least show the link
+	 * file pointing to when reporting unsupported file.
+	 * On error simply empty string here.
+	 */
+	if (fill_fdlink(lfd, &p, &link))
+		memzero(&link, sizeof(link));
+
+	return dump_unsupp_fd(&p, lfd, img, "unknown", link.name + 1);
 }
 
 int dump_task_files_seized(struct parasite_ctl *ctl, struct pstree_item *item,
@@ -993,11 +1000,7 @@ static int serve_out_fd(int pid, int fd, struct file_desc *d)
 	int sock, ret;
 	struct fdinfo_list_entry *fle;
 
-	sock = socket(PF_UNIX, SOCK_DGRAM, 0);
-	if (sock < 0) {
-		pr_perror("Can't create socket");
-		return -1;
-	}
+	sock = get_service_fd(TRANSPORT_FD_OFF);
 
 	pr_info("\t\tCreate fd for %d\n", fd);
 
@@ -1015,7 +1018,6 @@ static int serve_out_fd(int pid, int fd, struct file_desc *d)
 
 	ret = 0;
 out:
-	close(sock);
 	return ret;
 }
 
@@ -1665,3 +1667,21 @@ char *external_lookup_by_key(char *key)
 	}
 	return NULL;
 }
+
+int open_transport_socket()
+{
+	int sock;
+
+	sock = socket(PF_UNIX, SOCK_DGRAM | SOCK_CLOEXEC, 0);
+	if (sock < 0) {
+		pr_perror("Can't create socket");
+		return -1;
+	}
+	if (install_service_fd(TRANSPORT_FD_OFF, sock) < 0) {
+		close(sock);
+		return -1;
+	}
+	close(sock);
+
+	return 0;
+}
