@@ -418,8 +418,6 @@ static int parasite_check_aios(struct parasite_check_aios_args *args)
 		}
 
 		/* XXX: wait aio completion */
-
-		args->ring[i].max_reqs = ring->nr;
 	}
 
 	return 0;
@@ -737,7 +735,7 @@ static noinline __used int parasite_init_daemon(void *data)
 	args->sigreturn_addr = fini_sigreturn;
 	sigframe = args->sigframe;
 
-	tsock = sys_socket(PF_UNIX, SOCK_SEQPACKET, 0);
+	ret = tsock = sys_socket(PF_UNIX, SOCK_SEQPACKET, 0);
 	if (tsock < 0) {
 		pr_err("Can't create socket: %d\n", tsock);
 		goto err;
@@ -749,6 +747,8 @@ static noinline __used int parasite_init_daemon(void *data)
 		goto err;
 	}
 
+	futex_set_and_wake(&args->daemon_connected, 1);
+
 	ret = recv_fd(tsock);
 	if (ret >= 0) {
 		log_set_fd(ret);
@@ -760,6 +760,7 @@ static noinline __used int parasite_init_daemon(void *data)
 	parasite_daemon(data);
 
 err:
+	futex_set_and_wake(&args->daemon_connected, ret);
 	fini();
 	BUG();
 
